'use strict';

/* eslint-disable new-cap */
/* eslint-disable-line no-param-reassign */

// This has been completely stolen from @threepointone's amazing original
// implementation which can be found here:
// https://github.com/threepointone/react-modules
// Go look at his implementation! It is far more featureful.
// All props go to Sunil! Love love love.

var template = require('babel-template');

// -----------------------------------------------------------------------------
// PRIVATES

var CODE_SPLIT_COMPONENT_NAME = 'CodeSplitComponent';

var MISSING_PROP = 'A CodeSplitComponent must have either a "modules" prop or a "module" prop assigned';

var INVALID_MODULE_PROP_VALUE = '\nYou must supply a single require statement containing a string literal to the "module" property of the "CodeSplitComponent".\n\nFor example:\n\n  <CodeSplitComponent module={require(\'./Foo\')}>\n    {\n      Foo => Foo\n        ? <Foo />\n        : <span>Loading..</span>\n    }\n  </CodeSplitComponentLoader>';

var INVALID_MODULES_PROP_VALUE = '\nYou must supply an array containing at least one require statement within to the "modules" prop of the "CodeSplitComponent".  Each require statement within the array must have string literal value.\n\nFor example:\n\n  <CodeSplitComponent modules={[require(\'./Foo\'), require(\'./Bar\')]}>\n    {\n      [Foo, Bar] => Foo && Bar\n        ? <div><Foo /><Bar /></div>\n        : <span>Loading..</span>\n    }\n  </CodeSplitComponentLoader>';

var err = function err(error) {
  return '\n\nERROR IN CodeSplitComponent BABEL PLUGIN TRANSPILATION ATTEMPT\n--------------------------------------------------------------\n' + error + '\n\n';
};

var SYSTEM_IMPORT = template('System.import(SOURCE)');
var PROMISE_ALL = template('Promise.all(SOURCE)');

var TRUE = template('true');

var isValidPropType = function isValidPropType(element) {
  return element.type && element.value.type === 'JSXExpressionContainer' && element.value.expression;
};

var isValidElement = function isValidElement(element) {
  return element.type === 'CallExpression' && element.callee && element.callee.type === 'Identifier' && element.callee.name === 'require' && element.arguments && element.arguments.length > 0 && element.arguments[0].type === 'StringLiteral';
};

var isInvalidElement = function isInvalidElement(element) {
  return !isValidElement(element);
};

var isValidModulesExpression = function isValidModulesExpression(expression) {
  return expression.type === 'ArrayExpression' && expression.elements.length > 0 && expression.elements.findIndex(isInvalidElement) === -1;
};

// -----------------------------------------------------------------------------
// PLUGIN

function codeSplitComponentPlugin(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      JSXElement: function JSXElement(path, state) {
        if (state.opts.noCodeSplitting) {
          return;
        }

        if (path.node.openingElement.name.name === CODE_SPLIT_COMPONENT_NAME) {
          var moduleProp = path.node.openingElement.attributes.find(function (attr) {
            return attr.name && attr.name.name === 'module';
          });

          var modulesProp = path.node.openingElement.attributes.find(function (attr) {
            return attr.name && attr.name.name === 'modules';
          });

          if (!moduleProp && !modulesProp) {
            throw new Error(err(MISSING_PROP));
          }

          if (moduleProp) {
            var expression = moduleProp.value.expression;

            if (!isValidPropType(moduleProp) || !isValidElement(expression)) {
              throw new Error(err(INVALID_MODULE_PROP_VALUE));
            }

            var modulePath = expression.arguments[0].value;
            var newExpression = SYSTEM_IMPORT({
              SOURCE: t.stringLiteral(modulePath)
            }).expression;

            moduleProp.value = t.jSXExpressionContainer(newExpression);
          } else {
            // modules prop

            var _expression = modulesProp.value.expression;

            if (!isValidPropType(modulesProp) || !isValidModulesExpression(_expression)) {
              throw new Error(err(INVALID_MODULES_PROP_VALUE));
            }

            var requireStatements = _expression.elements;
            var count = requireStatements.length;

            var vals = requireStatements.map(function (moduleRequire) {
              var modulePath = moduleRequire.arguments[0].value;
              return SYSTEM_IMPORT({ SOURCE: t.stringLiteral(modulePath) }).expression;
            });

            var _newExpression = PROMISE_ALL({
              SOURCE: t.arrayExpression(vals)
            }).expression;

            modulesProp.value = t.jSXExpressionContainer(_newExpression);

            // Adds a moduleCount={x} prop to the code split component.
            path.node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('moduleCount'), t.jSXExpressionContainer(t.numericLiteral(count))));
          }

          // Adds a noCodeSplitting={true|false} prop to the code split component.
          path.node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('codeSplit'), t.jSXExpressionContainer(TRUE().expression)));
        }
      }
    }
  };
}

module.exports = codeSplitComponentPlugin;